
// AnimCanvas.java
// Andrew Davison, dandrew@ratree.psu.ac.th, October 2004

/* Create a 3D scene using retained mode (i.e. a scenegraph).
   The scenegraph contains a camera, lights, a blue background,
   a floor (made by Floor), and an animated model (made by
   AnimModel).

   The scene is updated and repainted by a timer task in 
   AnimM3G which calls update() periodically.

   ===============
   Changes. 26th Oct. 2004

   * added loadImage() which uses Loader.load()
       - use it to load the floor and background images;

   * moved the floor creation code to its own addFloor() method;

   * moved the floor texture and size information to addFloor()
     rather than storing them as constants;

   * added bigGrid.gif and grass.gif as alternative floor images.
*/


import javax.microedition.lcdui.*;
import javax.microedition.m3g.*;


public class AnimCanvas extends Canvas implements CommandListener 
{
  private Command exitCmd; 

  private Graphics3D iG3D;

  private AnimM3G top;
  private World scene;
  private AnimModel animModel;

  // timing information
  private int appTime = 0;
  private int nextTimeToAnimate;



  public AnimCanvas(AnimM3G top) 
  { 
    this.top = top;

    exitCmd = new Command("Exit", Command.EXIT, 0);
    addCommand(exitCmd);
    setCommandListener(this);

    iG3D = Graphics3D.getInstance();
    scene = new World();   // create scene graph
    buildScene();

    // start the animation
    nextTimeToAnimate = scene.animate(appTime);
  }  // end of AnimCanvas()


  public void commandAction(Command command, Displayable disp)
  { 
    if (command == exitCmd)    // finish the animation
      top.finishGame();
    else   // execution should not get here
      System.out.println("Command Action Error: " + command);
  }


  private void buildScene()
  // add elements to the scene
  {
    addCamera();
    addLight();
    addBackground();

    animModel = new AnimModel();
    scene.addChild( animModel.getModelGroup() );   // add the model

    addFloor();
  } // end of buildScene()


  private void addCamera()
  {
    Camera cam = new Camera();
    float aspectRatio = ((float) getWidth()) / ((float) getHeight());
    cam.setPerspective(70.0f, aspectRatio, 0.1f, 50.0f);

    // look straight down from above
    // cam.setTranslation(0.0f, 5.0f, 0.0f);
    // cam.setOrientation(-90.0f, 1.0f, 0, 0);

    cam.setTranslation(0.0f, 0.5f, 2.0f);   // up and back
    // cam.setOrientation(-10.0f, 1.0f, 0, 0); // angle downwards slightly

    scene.addChild(cam);
    scene.setActiveCamera(cam); 
  }  // end of addCamera()


  private void addLight()
  {
    Light light = new Light();  // default white, directional light
    light.setIntensity(1.25f);  // make it a bit brighter
    light.setOrientation(-45.0f, 1.0f, 0, 0); // pointing down and into scene

    scene.addChild(light);
  }  // end of addLight()


  private void addBackground()
  { Background backGnd = new Background();
    // backGnd.setColor(0x00bffe); // a light blue background

   Image2D backIm = loadImage("/clouds.gif");
    if (backIm != null) {
      backGnd.setImage(backIm);
      backGnd.setImageMode(Background.REPEAT, Background.REPEAT);
    }
    else 
      backGnd.setColor(0x00bffe); // a light blue background

    scene.setBackground(backGnd);
  } // end of addBackground()


  private void addFloor()
  {
    // Image2D floorIm = loadImage("/bigGrid.gif");   // large, so slow to load
    // Floor f = new Floor( floorIm, 8);   // 8 by 8 size

    Image2D floorIm = loadImage("/grass.gif");    // or try "/floor.png"
    Floor f = new Floor( floorIm, 6);   // 6 by 6 size

    scene.addChild( f.getFloorMesh() );
  }  // end of addFloor()


  private Image2D loadImage(String fn)
  // load the image stored in fn
  { Image2D im = null;
    try {
       im = (Image2D)Loader.load(fn)[0];
    }
    catch (Exception e)
    { System.out.println("Cannot make image from " + fn); }
    return im;
  }  // end of loadImage()


  // ------------------ update and paint the canvas ------------------

  public void update()
  // called by a timer task in AnimM3G
  {
    appTime++;
    if (appTime >= nextTimeToAnimate) {
      nextTimeToAnimate = scene.animate(appTime) + appTime;
      // System.out.println("nextTimeToAnimate: " + nextTimeToAnimate);
      repaint();
    }
  }  // end of update()


  protected void paint(Graphics g)
  /* Painting mixes 3D and 2D: first the 3D scene, 
     then model coordinates are shown using drawString. */
  {
    iG3D.bindTarget(g);
    try {
      iG3D.render(scene);
    }
    catch(Exception e)
    { e.printStackTrace(); }
    finally {
      iG3D.releaseTarget();
    }

    // show the model's coordinates
    g.drawString( animModel.getPosition(), 5,5, 
                            Graphics.TOP|Graphics.LEFT);
  } // end of paint()

} // end of AnimCanvas class
