
// AnimModel.java
// Andrew Davison, dandrew@ratree.psu.ac.th, October 2004

/* Create a 3D model which moves and rotates in a circle.

   The model data was generated automatically from ObjView,
   and pasted into this class.

   The model's current position can be accessed using 
   getPosition()

   The model is beneath a Group node:
       transRotGroup --> model

   transRotGroup is used by the animation controller to move 
   and rotate the model. The affect is equivalent to
      New_Position = Trans * Rot * Old_Position
   (See the Transformable documentation for details; Group is
    a subclass.)

   The rotations are carried out using quaternions.
   A nice intro to quaternions with a gaming slant:
     http://www.gamasutra.com/features/19980703/quaternions_01.htm

   transRotGroup is returned by getModelGroup(), and is connected
   to the rest of the scene graph in buildScene() in AnimCanvas.

   ===============
   Changes. 26th Oct. 2004

   * adjusted the model's position with setTranslation() in 
     AnimModel() so it's centered on the origin

*/


import javax.microedition.lcdui.*;
import javax.microedition.m3g.*;


public class AnimModel 
{
  private static final String MODEL_TEXTURE = "/model.gif";

  // for translating and rotating the model
  private Group transRotGroup;

  // for examining the model's position
  private float transMat[] = new float[16];
  private Transform modelTransform = new Transform();
  private float xCoord, yCoord, zCoord;



  public AnimModel() 
  { 
    // dummy initial model position
    xCoord = 0.0f; yCoord = 0.0f; zCoord = 0.0f;

    Mesh model = makeModel();

  /* Add the translation/rotation group
     transRotGroup --> model
  */
    // reposition the model's start position and size
    model.setTranslation(0.25f, 0.25f, 0.25f); // so at center
    model.scale(0.5f, 0.5f, 0.5f);

    // translation/rotation group for the model
    transRotGroup = new Group();
    transRotGroup.addChild(model);

    setUpAnimation();
  }  // end of AnimModel()



  private void setUpAnimation()
  {
    // creation animation controller 
    AnimationController animController = new AnimationController();
    animController.setActiveInterval(0, 1500);
    // animController.setSpeed(2.0f, 0);	

    // creation translation animation track 
    KeyframeSequence transKS = translateFrames();
    AnimationTrack transTrack = 
            new AnimationTrack(transKS, AnimationTrack.TRANSLATION); 
    transTrack.setController(animController);
    transRotGroup.addAnimationTrack(transTrack);  // will affect transRotGroup

    // creation rotation animation track 
    KeyframeSequence rotKS = rotationFrames();
    AnimationTrack rotTrack = 
           new AnimationTrack(rotKS, AnimationTrack.ORIENTATION); 
    rotTrack.setController(animController);
    transRotGroup.addAnimationTrack(rotTrack);   // will affect transRotGroup
  }  // end of setUpAnimation()


  private KeyframeSequence translateFrames()
  /* One complete circular translation is carried out over 8 
     frames, then repeated. Each sequence taking 80 time units.

     The coordinates describe a circle of radius 0.5f, centered
     on the origin on the XZ plane. 
     The starting point is at -0.5f on the x-axis, which is on
     the left of the origin.
  */
  {
    KeyframeSequence ks = new KeyframeSequence(8, 3, KeyframeSequence.SPLINE);
        /* Use a spline to interpolated between the points, so the 
           movement is in a curve (actually a circle) */

    // move clockwise in a circle;
    // each frame is separated by 10 sequence time units
    ks.setKeyframe(0, 0, new float[] { -0.5f, 0.0f, 0.0f });
    ks.setKeyframe(1, 10, new float[] { -0.3536f, 0.0f, 0.3536f });
    ks.setKeyframe(2, 20, new float[] { 0.0f, 0.0f, 0.5f });
    ks.setKeyframe(3, 30, new float[] { 0.3536f, 0.0f, 0.3536f });
    ks.setKeyframe(4, 40, new float[] { 0.5f, 0.0f, 0.0f });
    ks.setKeyframe(5, 50, new float[] { 0.3536f, 0.0f, -0.3536f });
    ks.setKeyframe(6, 60, new float[] { 0.0f, 0.0f, -0.5f });
    ks.setKeyframe(7, 70, new float[] { -0.3536f, 0.0f, -0.3536f });

    ks.setDuration(80);   // one cycle takes 80 sequence time units
    ks.setValidRange(0, 7);
    ks.setRepeatMode(KeyframeSequence.LOOP);

    return ks;
  }  // end of translateFrames()



  private KeyframeSequence rotationFrames()
  /* One complete rotation is carried out over 8 frames, then repeated.
     Each sequence taking 80 time units.

     The number of keyframes and their duration are the same as the
     translation sequence, so the rotation will keep the model facing
     'forward' at a tangent to the model's current position on its
     circular path. 
  */
  {
    KeyframeSequence ks = new KeyframeSequence(8, 4, KeyframeSequence.SLERP);
      /* the interpolation is a constant speed rotation */

    // rotate clockwise in a complete loop;
    // each frame is separated by 10 sequence time units
    ks.setKeyframe(0, 0, rotYQuat(0));
    ks.setKeyframe(1, 10, rotYQuat(45));
    ks.setKeyframe(2, 20, rotYQuat(90));
    ks.setKeyframe(3, 30, rotYQuat(135));
    ks.setKeyframe(4, 40, rotYQuat(180));
    ks.setKeyframe(5, 50, rotYQuat(225));
    ks.setKeyframe(6, 60, rotYQuat(270));
    ks.setKeyframe(7, 70, rotYQuat(315));

    ks.setDuration(80);   // one cycle takes 80 sequence time units
    ks.setValidRange(0, 7);
    ks.setRepeatMode(KeyframeSequence.LOOP);

    return ks;
  }  // end of rotationFrames()


  private float[] rotYQuat(double angle)
  /* Calculate the quaternion for a clockwise rotation of 
     angle degress about the y-axis. */
  {
    double radianAngle = Math.toRadians(angle)/2.0;
    float[] quat = new float[4];
    quat[0] = 0.0f;    // i coefficient
    quat[1] = (float) Math.sin(radianAngle);   // j coef
    quat[2] = 0.0f;    // k coef
    quat[3] = (float) Math.cos(radianAngle);   // scalar component a
    return quat;
  }  // end of rotYQuaternion()



 // ----------- access methods ----------------

  public Group getModelGroup()
  {  return transRotGroup;  }


  public String getPosition()
  // get the current (x,y,z) position of the model
  {
    transRotGroup.getCompositeTransform(modelTransform);
    modelTransform.get(transMat);

    // store coords rounded to 2 dp
    xCoord = ((int)((transMat[3]+0.005)*100.0f))/100.0f;
    yCoord = ((int)((transMat[7]+0.005)*100.0f))/100.0f;
    zCoord = ((int)((transMat[11]+0.005)*100.0f))/100.0f;

    return "Posn: (" + xCoord + ", " + yCoord + ", " + zCoord + ")";

/*  System.out.print("getTransform");
    for(int i=0; i < 16; i++) {
      if (i%4 == 0)
        System.out.println();
      System.out.print(transMat[i] + " ");
    }
    System.out.println();
*/
  }  // end of getPosition()



  // ------------------- model creation -----------------------------

  private Mesh makeModel()
  {
    VertexBuffer modelVertBuf = makeGeometry();
    IndexBuffer modelIdxBuf = 
           new TriangleStripArray(0, getStripLengths());
    Appearance modelApp = makeAppearance(MODEL_TEXTURE);

    Mesh m = new Mesh(modelVertBuf, modelIdxBuf, modelApp);
    return m;
  }  // end of makeModel()


  private VertexBuffer makeGeometry()
  {
    // create vertices
    short[] verts = getVerts();
    VertexArray va = new VertexArray(verts.length / 3, 3, 2);
    va.set(0, verts.length / 3, verts);

    // create normals
    byte[] norms = getNormals();
    VertexArray normArray = new VertexArray(norms.length / 3, 3, 1);
    normArray.set(0, norms.length / 3, norms);

    // create texture coordinates
    short[] tcs = getTexCoordsRev();
    VertexArray texArray = new VertexArray(tcs.length / 2, 2, 2);
    texArray.set(0, tcs.length / 2, tcs);

    float[] pbias = { (1.0f / 255.0f), (1.0f / 255.0f), (1.0f / 255.0f)};

    VertexBuffer vb = new VertexBuffer(); 
    vb.setPositions(va, (2.0f / 255.0f), pbias); // scale, bias
    vb.setNormals(normArray);
    vb.setTexCoords(0, texArray, (1.0f / 255.0f), null);

    return vb;
  }  // end of makeGeometry()


  private short[] getTexCoordsRev()
  { 
    short[] tcs = getTexCoords();
    // t' = 255 - t   (will later be scaled from 255 to 1)
    for (int i = 1; i < tcs.length; i = i + 2)
      tcs[i] = (short) (255 - tcs[i]);

    return tcs;
  }  // end of getTexCoordsRev()


  private Appearance makeAppearance(String fn)
  {
    Appearance app = new Appearance();
    app.setMaterial( setMatColours() );

    Image im = loadImage(fn);
    if (im != null) {
      // create an Image2D for the Texture2D
      Image2D image2D = new Image2D(Image2D.RGB, im);

      // create the Texture2D and enable mipmapping
      Texture2D tex = new Texture2D(image2D);

      tex.setFiltering(Texture2D.FILTER_NEAREST, Texture2D.FILTER_NEAREST);
      tex.setWrapping(Texture2D.WRAP_CLAMP, Texture2D.WRAP_CLAMP);
      tex.setBlending(Texture2D.FUNC_MODULATE);  // mix material and texture

      app.setTexture(0, tex);
    }
    return app;
  }  // end of makeAppearance()



  private Image loadImage(String fn)
  // load the image stored in fn
  { Image im = null;
    try {
      im = Image.createImage(fn);
    }
    catch (Exception e)
    { System.out.println("Cannot make image from " + fn); }
    return im;
  }  // end of loadImage()


  // ------------------ model data ------------------------

// Methods for model in penguin.obj

  private short[] getVerts()
  // return an array holding Verts [870 values / 3 = 290 points] 
  {
    short[] vals = {
      	-60,-32,-105,  	-60,-3,-105,  	-38,-32,-99,  	-38,-3,-99,  
	-38,-32,-99,  	-22,13,-84,  	-22,-32,-83,  	-22,13,-84,  
	-17,-32,-60,  	-17,13,-61,  	-17,-32,-60,  	-22,13,-39,  
	-22,-32,-38,  	-22,13,-39,  	-38,-32,-22,  	-38,-3,-22,  
	-38,-32,-22,  	-60,-3,-16,  	-60,-32,-16,  	-60,-3,-16,  
	-81,-32,-22,  	-81,-3,-22,  	-81,-32,-22,  	-95,18,-45,  
	-97,-32,-38,  	-95,18,-45,  	-103,-32,-60,  	-99,11,-60,  
	-103,-32,-60,  	-93,18,-89,  	-97,-32,-83,  	-93,18,-89,  
	-81,-32,-99,  	-93,18,-89,  	-81,-3,-99,  	-93,18,-89,  
	-81,25,-99,  	-122,7,-83,  	-81,25,-99,  	-97,54,-83,  
	-81,54,-99,  	-97,54,-83,  	-81,82,-99,  	-97,54,-83,  
	-97,82,-83,  	-97,54,-83,  	-107,82,-60,  	-103,54,-60,  
	-107,82,-60,  	-97,54,-38,  	-97,82,-38,  	-97,54,-38,  
	-81,82,-22,  	-81,54,-22,  	-81,82,-22,  	-59,64,9,  
	-59,82,-16,  	-59,64,9,  	-37,82,-22,  	-37,54,-22,  
	-37,82,-22,  	-22,54,-38,  	-22,82,-38,  	-22,54,-38,  
	-17,82,-60,  	-17,54,-60,  	-17,82,-60,  	-22,54,-83,  
	-22,82,-83,  	-22,54,-83,  	-38,82,-99,  	-22,54,-83,  
	-38,54,-99,  	-22,54,-83,  	-38,25,-99,  	-22,54,-83,  
	9,1,-76,  	-22,54,-83,  	9,1,-54,  	-17,54,-60,  
	9,1,-54,  	-22,54,-38,  	-22,25,-38,  	-22,54,-38,  
	-37,25,-22,  	-37,54,-22,  	-37,25,-22,  	-59,64,9,  
	-59,25,-16,  	-59,64,9,  	-81,25,-22,  	-81,54,-22,  
	-81,25,-22,  	-97,54,-38,  	-97,25,-38,  	-97,54,-38,  
	-128,7,-60,  	-103,54,-60,  	-128,7,-60,  	-97,54,-83,  
	-128,7,-60,  	-122,7,-83,  	-128,7,-60,  	-93,18,-89,  
	-128,7,-60,  	-99,11,-60,  	-128,7,-60,  	-95,18,-45,  
	-97,25,-38,  	-95,18,-45,  	-81,25,-22,  	-81,-3,-22,  
	-81,25,-22,  	-60,-3,-16,  	-59,25,-16,  	-60,-3,-16,  
	-37,25,-22,  	-38,-3,-22,  	-37,25,-22,  	-22,13,-39,  
	-22,25,-38,  	-22,13,-39,  	9,1,-54,  	-17,13,-61,  
	9,1,-54,  	-22,13,-84,  	9,1,-76,  	-22,13,-84,  
	-38,25,-99,  	-38,-3,-99,  	-38,25,-99,  	-60,-3,-105,  
	-38,25,-99,  	-60,25,-105,  	-38,25,-99,  	-60,54,-105,  
	-38,54,-99,  	-60,54,-105,  	-38,82,-99,  	-60,82,-105,  
	-38,82,-99,  	-60,111,-105,  	-38,82,-99,  	-38,111,-99,  
	-38,82,-99,  	-22,111,-83,  	-22,82,-83,  	-22,111,-83,  
	-17,82,-60,  	-16,111,-60,  	-17,82,-60,  	-22,111,-38,  
	-22,82,-38,  	-22,111,-38,  	-37,82,-22,  	-37,111,-22,  
	-37,82,-22,  	-59,111,-16,  	-59,82,-16,  	-59,111,-16,  
	-81,82,-22,  	-81,111,-22,  	-81,82,-22,  	-97,111,-38,  
	-97,82,-38,  	-97,111,-38,  	-107,82,-60,  	-107,111,-61,  
	-107,82,-60,  	-97,111,-84,  	-97,82,-83,  	-97,111,-84,  
	-81,82,-99,  	-81,111,-100,  	-81,82,-99,  	-60,111,-105,  
	-81,82,-99,  	-60,82,-105,  	-81,82,-99,  	-60,54,-105,  
	-81,54,-99,  	-60,54,-105,  	-81,25,-99,  	-60,25,-105,  
	-81,25,-99,  	-60,-3,-105,  	-81,-3,-99,  	-60,-3,-105,  
	-81,-32,-99,  	-60,-32,-105,  	-81,-32,-99,  	-60,-60,-105,  
	-81,-32,-99,  	-81,-60,-99,  	-81,-32,-99,  	-97,-60,-83,  
	-97,-32,-83,  	-97,-60,-83,  	-103,-32,-60,  	-107,-60,-60,  
	-103,-32,-60,  	-116,-60,-26,  	-97,-32,-38,  	-116,-60,-26,  
	-81,-32,-22,  	-108,-60,-19,  	-81,-32,-22,  	-97,-60,-17,  
	-81,-32,-22,  	-77,-60,-14,  	-81,-32,-22,  	-60,-60,-16,  
	-60,-32,-16,  	-60,-60,-16,  	-38,-32,-22,  	-40,-60,-14,  
	-38,-32,-22,  	-18,-60,-14,  	-38,-32,-22,  	-7,-60,-18,  
	-38,-32,-22,  	-4,-61,-27,  	-22,-32,-38,  	-4,-61,-27,  
	-17,-32,-60,  	-11,-60,-60,  	-17,-32,-60,  	-22,-60,-83,  
	-22,-32,-83,  	-22,-60,-83,  	-38,-32,-99,  	-38,-60,-99,  
	-38,-32,-99,  	-60,-60,-105,  	-60,-32,-105,  	-60,111,-105,  
	-81,111,-100,  	-60,127,-60,  	-97,111,-84,  	-60,127,-60,  
	-107,111,-61,  	-60,127,-60,  	-97,111,-38,  	-60,127,-60,  
	-81,111,-22,  	-60,127,-60,  	-59,111,-16,  	-60,127,-60,  
	-37,111,-22,  	-60,127,-60,  	-22,111,-38,  	-60,127,-60,  
	-16,111,-60,  	-60,127,-60,  	-22,111,-83,  	-60,127,-60,  
	-38,111,-99,  	-60,111,-105,  	-60,-60,-105,  	-38,-60,-99,  
	-60,-60,-60,  	-22,-60,-83,  	-60,-60,-60,  	-11,-60,-60,  
	-60,-60,-60,  	-4,-60,-27,  	-60,-60,-60,  	-7,-60,-19,  
	-60,-60,-60,  	-18,-60,-14,  	-60,-60,-60,  	-40,-60,-14,  
	-60,-60,-60,  	-60,-60,-16,  	-60,-60,-60,  	-77,-60,-14,  
	-60,-60,-60,  	-97,-60,-17,  	-60,-60,-60,  	-108,-60,-19,  
	-60,-60,-60,  	-116,-60,-26,  	-60,-60,-60,  	-107,-60,-60,  
	-60,-60,-60,  	-97,-60,-83,  	-60,-60,-60,  	-81,-60,-99,  
	-60,-60,-60,  	-60,-60,-105      };
    return vals;
  }  // end of getVerts()


  private byte[] getNormals()
  // return an array holding Normals [870 values / 3 = 290 points] 
  {
    byte[] vals = {
      	34,0,-123,  	34,0,-123,  	65,0,-110,  	64,0,-111,  
	65,0,-110,  	77,-69,-76,  	112,-1,-63,  	77,-69,-76,  
	126,16,-12,  	66,-110,1,  	126,16,-12,  	98,-20,80,  
	106,41,59,  	98,-20,80,  	53,33,112,  	67,0,109,  
	53,33,112,  	0,0,127,  	0,0,127,  	0,0,127,  
	-52,30,113,  	-78,1,102,  	-52,30,113,  	-96,-39,75,  
	-104,48,58,  	-96,-39,75,  	-126,20,-11,  	-89,-91,13,  
	-126,20,-11,  	-78,-65,-78,  	-112,0,-63,  	-78,-65,-78,  
	-64,-1,-111,  	-78,-65,-78,  	-59,0,-114,  	-78,-65,-78,  
	-47,-11,-118,  	-48,-73,-94,  	-47,-11,-118,  	-106,18,-69,  
	-65,0,-110,  	-106,18,-69,  	-65,-2,-111,  	-106,18,-69,  
	-106,-2,-72,  	-106,18,-69,  	-128,-5,1,  	-126,22,0,  
	-128,-5,1,  	-103,5,76,  	-105,1,73,  	-103,5,76,  
	-76,14,102,  	-98,0,82,  	-76,14,102,  	0,13,127,  
	0,59,114,  	0,13,127,  	77,14,101,  	100,0,80,  
	77,14,101,  	105,8,73,  	112,-1,61,  	105,8,73,  
	127,-1,0,  	125,29,0,  	127,-1,0,  	110,19,-63,  
	112,0,-63,  	110,19,-63,  	65,0,-110,  	110,19,-63,  
	65,0,-110,  	110,19,-63,  	50,-12,-117,  	110,19,-63,  
	54,-76,-88,  	110,19,-63,  	88,-57,73,  	125,29,0,  
	88,-57,73,  	105,8,73,  	75,-4,104,  	105,8,73,  
	77,-10,102,  	100,0,80,  	77,-10,102,  	0,13,127,  
	0,-37,123,  	0,13,127,  	-77,-21,100,  	-98,0,82,  
	-77,-21,100,  	-103,5,76,  	-68,-48,97,  	-103,5,76,  
	-90,-73,55,  	-126,22,0,  	-90,-73,55,  	-106,18,-69,  
	-90,-73,55,  	-48,-73,-94,  	-90,-73,55,  	-78,-65,-78,  
	-90,-73,55,  	-89,-91,13,  	-90,-73,55,  	-96,-39,75,  
	-68,-48,97,  	-96,-39,75,  	-77,-21,100,  	-78,1,102,  
	-77,-21,100,  	0,0,127,  	0,-37,123,  	0,0,127,  
	77,-10,102,  	67,0,109,  	77,-10,102,  	98,-20,80,  
	75,-4,104,  	98,-20,80,  	88,-57,73,  	66,-110,1,  
	88,-57,73,  	77,-69,-76,  	54,-76,-88,  	77,-69,-76,  
	50,-12,-117,  	64,0,-111,  	50,-12,-117,  	34,0,-123,  
	50,-12,-117,  	34,0,-123,  	50,-12,-117,  	34,0,-123,  
	65,0,-110,  	34,0,-123,  	65,0,-110,  	34,0,-123,  
	65,0,-110,  	34,0,-123,  	65,0,-110,  	65,0,-110,  
	65,0,-110,  	111,0,-63,  	112,0,-63,  	111,0,-63,  
	127,-1,0,  	127,-2,-1,  	127,-1,0,  	112,-1,62,  
	112,-1,61,  	112,-1,62,  	77,14,101,  	66,0,110,  
	77,14,101,  	0,0,127,  	0,59,114,  	0,0,127,  
	-76,14,102,  	-65,0,110,  	-76,14,102,  	-105,1,73,  
	-105,1,73,  	-105,1,73,  	-128,-5,1,  	-128,0,2,  
	-128,-5,1,  	-106,-3,-71,  	-106,-2,-72,  	-106,-3,-71,  
	-65,-2,-111,  	-62,-5,-112,  	-65,-2,-111,  	-31,-3,-124,  
	-65,-2,-111,  	-34,0,-123,  	-65,-2,-111,  	-34,0,-123,  
	-65,0,-110,  	-34,0,-123,  	-47,-11,-118,  	-34,0,-123,  
	-47,-11,-118,  	-34,0,-123,  	-59,0,-114,  	-34,0,-123,  
	-64,-1,-111,  	-34,0,-123,  	-64,-1,-111,  	-34,0,-123,  
	-64,-1,-111,  	-65,0,-110,  	-64,-1,-111,  	-109,4,-67,  
	-112,0,-63,  	-109,4,-67,  	-126,20,-11,  	-120,18,-42,  
	-126,20,-11,  	-101,68,39,  	-104,48,58,  	-101,68,39,  
	-52,30,113,  	-52,58,102,  	-52,30,113,  	-22,36,121,  
	-52,30,113,  	0,35,123,  	-52,30,113,  	1,18,127,  
	0,0,127,  	1,18,127,  	53,33,112,  	-6,33,123,  
	53,33,112,  	19,45,118,  	53,33,112,  	66,80,75,  
	53,33,112,  	103,72,27,  	106,41,59,  	103,72,27,  
	126,16,-12,  	119,22,-42,  	126,16,-12,  	108,5,-69,  
	112,-1,-63,  	108,5,-69,  	65,0,-110,  	65,0,-110,  
	65,0,-110,  	34,0,-123,  	34,0,-123,  	1,120,-45,  
	-22,120,-39,  	1,127,0,  	-38,119,-26,  	1,127,0,  
	-42,121,0,  	1,127,0,  	-38,119,26,  	1,127,0,  
	-23,120,39,  	1,127,0,  	0,120,45,  	1,127,0,  
	23,120,39,  	1,127,0,  	40,120,22,  	1,127,0,  
	46,120,-1,  	1,127,0,  	40,119,-24,  	1,127,0,  
	23,120,-39,  	1,120,-45,  	0,-128,0,  	0,-128,0,  
	0,-128,0,  	0,-128,0,  	0,-128,0,  	0,-128,0,  
	0,-128,0,  	0,-128,0,  	0,-128,0,  	0,-128,0,  
	0,-128,0,  	0,-128,0,  	0,-128,0,  	0,-128,0,  
	0,-128,0,  	0,-128,0,  	0,-128,0,  	0,-128,0,  
	0,-128,0,  	0,-128,0,  	0,-128,0,  	0,-128,0,  
	0,-128,0,  	0,-128,0,  	0,-128,0,  	0,-128,0,  
	0,-128,0,  	0,-128,0,  	0,-128,0,  	0,-128,0,  
	0,-128,0,  	0,-128,0      };
    return vals;
  }  // end of getNormals()


  private short[] getTexCoords()
  // return an array holding TexCoords [580 values / 2 = 290 points] 
  {
    short[] vals = {
      	0,43,  	0,85,  	21,43,  	21,85,  
	21,43,  	43,85,  	43,43,  	43,85,  
	64,43,  	64,85,  	64,43,  	85,85,  
	85,43,  	85,85,  	107,43,  	107,85,  
	107,43,  	128,85,  	128,43,  	128,85,  
	149,43,  	149,85,  	149,43,  	171,85,  
	171,43,  	171,85,  	192,43,  	192,85,  
	192,43,  	213,85,  	213,43,  	213,85,  
	235,43,  	213,85,  	235,85,  	213,85,  
	235,128,  	213,128,  	235,128,  	213,171,  
	235,171,  	213,171,  	235,213,  	213,171,  
	213,213,  	213,171,  	192,213,  	192,171,  
	192,213,  	171,171,  	171,213,  	171,171,  
	149,213,  	149,171,  	149,213,  	128,171,  
	128,213,  	128,171,  	107,213,  	107,171,  
	107,213,  	85,171,  	85,213,  	85,171,  
	64,213,  	64,171,  	64,213,  	43,171,  
	43,213,  	43,171,  	21,213,  	43,171,  
	21,171,  	43,171,  	21,128,  	43,171,  
	43,128,  	43,171,  	64,128,  	64,171,  
	64,128,  	85,171,  	85,128,  	85,171,  
	107,128,  	107,171,  	107,128,  	128,171,  
	128,128,  	128,171,  	149,128,  	149,171,  
	149,128,  	171,171,  	171,128,  	171,171,  
	192,128,  	192,171,  	192,128,  	213,171,  
	192,128,  	213,128,  	192,128,  	213,85,  
	192,128,  	192,85,  	192,128,  	171,85,  
	171,128,  	171,85,  	149,128,  	149,85,  
	149,128,  	128,85,  	128,128,  	128,85,  
	107,128,  	107,85,  	107,128,  	85,85,  
	85,128,  	85,85,  	64,128,  	64,85,  
	64,128,  	43,85,  	43,128,  	43,85,  
	21,128,  	21,85,  	21,128,  	0,85,  
	21,128,  	0,128,  	21,128,  	0,171,  
	21,171,  	0,171,  	21,213,  	0,213,  
	21,213,  	0,255,  	21,213,  	21,255,  
	21,213,  	43,255,  	43,213,  	43,255,  
	64,213,  	64,255,  	64,213,  	85,255,  
	85,213,  	85,255,  	107,213,  	107,255,  
	107,213,  	128,255,  	128,213,  	128,255,  
	149,213,  	149,255,  	149,213,  	171,255,  
	171,213,  	171,255,  	192,213,  	192,255,  
	192,213,  	213,255,  	213,213,  	213,255,  
	235,213,  	235,255,  	235,213,  	255,255,  
	235,213,  	255,213,  	235,213,  	255,171,  
	235,171,  	255,171,  	235,128,  	255,128,  
	235,128,  	255,85,  	235,85,  	255,85,  
	235,43,  	255,43,  	235,43,  	255,0,  
	235,43,  	235,0,  	235,43,  	213,0,  
	213,43,  	213,0,  	192,43,  	192,0,  
	192,43,  	171,0,  	171,43,  	171,0,  
	149,43,  	165,0,  	149,43,  	160,0,  
	149,43,  	149,0,  	149,43,  	128,0,  
	128,43,  	128,0,  	107,43,  	107,0,  
	107,43,  	101,0,  	107,43,  	96,0,  
	107,43,  	85,0,  	85,43,  	85,0,  
	64,43,  	64,0,  	64,43,  	43,0,  
	43,43,  	43,0,  	21,43,  	21,0,  
	21,43,  	0,0,  	0,43,  	0,255,  
	0,255,  	0,255,  	0,255,  	0,255,  
	0,255,  	0,255,  	0,255,  	0,255,  
	0,255,  	0,255,  	0,255,  	0,255,  
	0,255,  	0,255,  	0,255,  	0,255,  
	0,255,  	0,255,  	0,255,  	0,255,  
	0,255,  	0,255,  	0,255,  	0,255,  
	0,255,  	0,255,  	0,255,  	0,255,  
	0,255,  	0,255,  	0,255,  	0,255,  
	0,255,  	0,255,  	0,255,  	0,255,  
	0,255,  	0,255,  	0,255,  	0,255,  
	0,255,  	0,255,  	0,255,  	0,255,  
	0,255,  	0,255,  	0,255,  	0,255,  
	0,255,  	0,255,  	0,255,  	0,255,  
	0,255,  	0,255      };
    return vals;
  }  // end of getTexCoords()


  private int[] getStripLengths()
  // return an array holding the lengths of each triangle strip
  {
    int[] lens = {
      235, 23, 32
    };
    return lens;
  }  // end of getStripLengths()


  private Material setMatColours()
  // set the material's colour and shininess values
  {
    Material mat = new Material();

    mat.setColor(Material.AMBIENT, 0x0003C1C1);
    mat.setColor(Material.EMISSIVE, 0x00000000);
    mat.setColor(Material.DIFFUSE, 0xFFE9B1E8);
    mat.setColor(Material.SPECULAR, 0x00000000);
    mat.setShininess(128.0f);

    return mat;
  } // end of setMatColours()


} // end of AnimModel class
